from typing import List

from pyopentdb.enum import response_code_map
from pyopentdb.model.QuestionResult import QuestionResult
from pyopentdb.model.QuestionSet import QuestionSet


class QuestionResponse:
    def __init__(self, response_code: int, results: List[dict]):
        self.response_code = response_code_map.get(response_code)
        self.results = QuestionSet([QuestionResult(**i).parse() for i in results])



from pyopentdb.enum.QuestionType import QuestionType, question_type_map
from pyopentdb.enum.Difficulty import Difficulty, difficulty_map
from pyopentdb.enum.Category import (
    Category,
    CategoryItem,
    category_id_map,
    category_name_map,
)
from pyopentdb.enum.ResponseCode import ResponseCode, response_code_map



from pyopentdb.model.Question import Question
from pyopentdb.model.QuestionResponse import QuestionResponse
from pyopentdb.model.QuestionResult import QuestionResult
from pyopentdb.model.QuestionSet import QuestionSet



from dataclasses import dataclass
from typing import List
from copy import deepcopy
import html

from pyopentdb.model.Question import Question
from pyopentdb.enum import (
    category_name_map,
    difficulty_map,
    question_type_map,
)
from pyopentdb.exc.QuestionError import QuestionError


@dataclass
class QuestionResult:
    category: str
    type: str
    difficulty: str
    question: str
    correct_answer: str
    incorrect_answers: List[str]

    def __post_init__(self):
        # Unescape all HTML encoded characters
        self.category = html.unescape(self.category)
        self.type = html.unescape(self.type)
        self.difficulty = html.unescape(self.difficulty)
        self.question = html.unescape(self.question)
        self.correct_answer = html.unescape(self.correct_answer)
        self.incorrect_answers = [html.unescape(i) for i in self.incorrect_answers]

    def parse(self) -> Question:
        # Parse values to enums
        category = category_name_map.get(self.category)
        question_type = question_type_map.get(self.type)
        difficulty = difficulty_map.get(self.difficulty)

        # Validate enums
        enums = {
            "Category": [category, self.category],
            "Question Type": [question_type, self.type],
            "Difficulty": [difficulty, self.difficulty],
        }
        for enum, [parsed, original] in enums.items():
            if parsed is None:
                raise QuestionError(f'Invalid {enum} "{original}"')

        # Build Question
        choices = deepcopy(self.incorrect_answers)
        choices.append(self.correct_answer)
        return Question(
            category=category,
            question_type=question_type,
            difficulty=difficulty,
            question=self.question,
            answer=self.correct_answer,
            choices=choices,
        )



import json
from copy import deepcopy
from typing import List, Union

from pyopentdb.model.Question import Question


class QuestionSet:
    def __init__(self, questions: List[Question]):
        self.items = questions

    def __len__(self) -> int:
        return len(self.items)

    def __getitem__(self, item: int) -> Question:
        return self.items[item]

    def __setitem__(self, key: int, value: Question) -> Question:
        self.items[key] = value
        return value

    def __delitem__(self, key) -> Question:
        item = deepcopy(self.items[key])
        del self.items[key]
        return item

    def __iter__(self) -> Question:
        yield from self.items

    def __str__(self):
        return str(self.items)

    def __repr__(self):
        return repr(self.items)

    def to_serializable(self, as_json: bool = False) -> Union[List[dict], str]:
        output = [i.to_serializable() for i in self.items]
        if as_json is True:
            return json.dumps(output)
        else:
            return output



from typing import Union
from warnings import warn

import requests

from pyopentdb.enum import Category, Difficulty, QuestionType, ResponseCode
from pyopentdb.exc import APIError
from pyopentdb.model import QuestionResponse, QuestionSet


class OpenTDBClient:
    def __init__(self):
        self.session = requests.Session()
        self.request_token()

    def request_token(self):
        req = requests.get(
            "https://opentdb.com/api_token.php", params={"command": "request"}
        )
        if req.ok:
            self.session.params.update({"token": req.json().get("token")})

    def reset_token(self):
        req = requests.get(
            f"https://opentdb.com/api_token.php",
            params={"command": "reset", "token": self.session.params.get("token")},
        )
        if req.ok:
            self.session.params.update({"token": req.json().get("token")})

    def get_questions(
        self,
        amount: int = 10,
        category: Union[Category, int] = None,
        difficulty: Union[Difficulty, str] = None,
        question_type: Union[QuestionType, str] = None,
        retry: int = 5,
    ) -> QuestionSet:
        # Validate amount
        if not 1 <= amount <= 50:
            raise APIError("Amount must be between 1 and 50, inclusive")

        # Parse enums
        if isinstance(category, Category):
            category = category.value.id
        if isinstance(difficulty, Difficulty):
            difficulty = difficulty.value
        if isinstance(question_type, QuestionType):
            question_type = question_type.value

        # Make HTTP request
        status = 200
        for i in range(retry + 1):
            req = self.session.get(
                "https://opentdb.com/api.php",
                params={
                    "amount": amount,
                    "category": category,
                    "difficulty": difficulty,
                    "type": question_type,
                },
            )
            if req.ok:
                resp = QuestionResponse(**req.json())
                if resp.response_code == ResponseCode.TOKEN_EMPTY:
                    self.reset_token()
                    continue
                elif resp.response_code == ResponseCode.TOKEN_NOT_FOUND:
                    self.request_token()
                    continue
                elif resp.response_code != ResponseCode.SUCCESS:
                    warn(resp.response_code.value.description)
                return resp.results
            else:
                status = req.status_code
                warn(
                    f"API returned status code {req.status_code}, retrying... ({i}/{retry})"
                )
                continue

        # Raise error if max retries exceeded
        raise APIError(
            f"API returned status code {status}, max retries exceeded, stopping."
        )

    def get_question_count(self, category: Union[Category, int] = None) -> dict:
        if category is None:
            req = self.session.get("https://opentdb.com/api_count_global.php")
        else:
            if isinstance(category, Category):
                category = category.value.id
            req = self.session.get(
                "https://opentdb.com/api_count.php", params={"category": category}
            )
        if req.ok:
            return req.json()



from enum import Enum, unique
from collections import namedtuple

ResponseCodeItem = namedtuple("ResponseCodeItem", ["code", "name", "description"])


@unique
class ResponseCode(Enum):
    SUCCESS = ResponseCodeItem(0, "Success", "Returned results successfully.")
    NO_RESULTS = ResponseCodeItem(
        1,
        "No Results",
        "Could not return results. The API doesn't have enough questions for your query.",
    )
    INVALID_PARAMETER = ResponseCodeItem(
        2,
        "Invalid Parameter",
        "Contains an invalid parameter. Arguments passed in are not valid.",
    )
    TOKEN_NOT_FOUND = ResponseCodeItem(
        3, "Token Not Found", "Session Token does not exist."
    )
    TOKEN_EMPTY = ResponseCodeItem(
        4,
        "Token Empty",
        "Session Token has returned all possible questions for the specified query. Resetting the Token is necessary.",
    )


response_code_map = {i.value.code: i for i in ResponseCode}



import json
from dataclasses import dataclass, asdict
from enum import Enum
from random import shuffle
from typing import List, Union

from pyopentdb.enum import QuestionType, Difficulty, Category
from pyopentdb.exc import QuestionError


@dataclass
class Question:
    category: Category
    question_type: QuestionType
    difficulty: Difficulty
    question: str
    choices: List[str]
    answer: str
    answer_index: int = None

    def __post_init__(self):
        # Shuffle choices
        shuffle(self.choices)

        # Try to set answer index
        try:
            self.answer_index = self.choices.index(self.answer)
        except ValueError:
            raise QuestionError(
                f"Answer ({self.answer}) is not in the list of choices ({self.choices})"
            )

    def to_serializable(self, as_json: bool = False) -> Union[dict, str]:
        output = {}
        for key, value in asdict(self).items():
            if isinstance(value, Category):
                output[key] = value.value.name
            elif isinstance(value, Enum):
                output[key] = value.value
            else:
                output[key] = value
        if as_json is True:
            return json.dumps(output)
        else:
            return output



from setuptools import setup, find_packages

import pyopentdb

with open("README.md", "r") as f:
    readme = f.read()

with open("requirements.txt", "r") as f:
    requirements = [i.rstrip() for i in f.readlines()]

setup(
    name="pyopentdb",
    version=pyopentdb.__version__,
    description=pyopentdb.__description__,
    long_description=readme,
    long_description_content_type="text/markdown",
    author=pyopentdb.__author__,
    author_email="andrew.mickael@gmail.com",
    license="MIT",
    platforms=["NT", "POSIX"],
    url="https://github.com/amickael/pyopentdb",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires=">=3.6",
)



from enum import Enum, unique
from collections import namedtuple

CategoryItem = namedtuple("CategoryItem", ["id", "name"])


@unique
class Category(Enum):
    GENERAL_KNOWLEDGE = CategoryItem(9, "General Knowledge")
    ENTERTAINMENT_BOOKS = CategoryItem(10, "Entertainment: Books")
    ENTERTAINMENT_FILM = CategoryItem(11, "Entertainment: Film")
    ENTERTAINMENT_MUSIC = CategoryItem(12, "Entertainment: Music")
    ENTERTAINMENT_MUSICALS_THEATRES = CategoryItem(
        13, "Entertainment: Musicals & Theatres"
    )
    ENTERTAINMENT_TELEVISION = CategoryItem(14, "Entertainment: Television")
    ENTERTAINMENT_VIDEO_GAMES = CategoryItem(15, "Entertainment: Video Games")
    ENTERTAINMENT_BOARD_GAMES = CategoryItem(16, "Entertainment: Board Games")
    SCIENCE_NATURE = CategoryItem(17, "Science & Nature")
    SCIENCE_COMPUTERS = CategoryItem(18, "Science: Computers")
    SCIENCE_MATHEMATICS = CategoryItem(19, "Science: Mathematics")
    MYTHOLOGY = CategoryItem(20, "Mythology")
    SPORTS = CategoryItem(21, "Sports")
    GEOGRAPHY = CategoryItem(22, "Geography")
    HISTORY = CategoryItem(23, "History")
    POLITICS = CategoryItem(24, "Politics")
    ART = CategoryItem(25, "Art")
    CELEBRITIES = CategoryItem(26, "Celebrities")
    ANIMALS = CategoryItem(27, "Animals")
    VEHICLES = CategoryItem(28, "Vehicles")
    ENTERTAINMENT_COMICS = CategoryItem(29, "Entertainment: Comics")
    SCIENCE_GADGETS = CategoryItem(30, "Science: Gadgets")
    ENTERTAINMENT_JAPANESE_ANIME_MANGA = CategoryItem(
        31, "Entertainment: Japanese Anime & Manga"
    )
    ENTERTAINMENT_CARTOON_ANIMATIONS = CategoryItem(
        32, "Entertainment: Cartoon & Animations"
    )


category_id_map = {i.value.id: i for i in Category}
category_name_map = {i.value.name: i for i in Category}



import requests

req = requests.get("https://opentdb.com/api_category.php")
data = req.json()["trivia_categories"]
for cat in data:
    ident = cat["id"]
    name = cat["name"]
    enum = name.upper().replace(" ", "_").replace(":", "_").replace("&", "_").strip()
    enum = enum.replace("___", "_").replace("__", "_")
    print(f'{enum} = CategoryItem({ident}, "{name}")')



from pyopentdb.api import OpenTDBClient
from pyopentdb.enum import (
    QuestionType,
    Difficulty,
    Category,
    CategoryItem,
    ResponseCode,
)
from pyopentdb.exc import APIError, QuestionError
from pyopentdb.model import Question

__author__ = "Andrew Mickael"
__version__ = "0.0.2"
__description__ = "Python interface for the Open Trivia DB"
